//+-----------------------------------------------------------------------------+
//| Copyright (C) 2013 REGEO (http://www.regeo.uji.es/)
//|
//| This program is free software
//| You can redistribute it and/or modify it under the terms of the GNU General Public License,
//| as published by the Free Software Foundation.
//| 
//| This program is distributed in the hope that it will be useful,
//| but WITHOUT ANY WARRANTY; without even the implied warranty of
//| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//+----------------------------------------------------------------------------*/

#ifndef _FITELLIPSE_
	#define _FITELLIPSE_

	class CFitEllipse
	{
		private:
			double ArcTolMin;
			double ArcTolMax;
			double ArcSmoothingPenalty;

			long StrokeCornersApproach;
			long FitEllipseApproach;
			long FitEllipseSpeed;
			bool UseConvexHull5P;
			bool CheckQuality5P;
			bool CheckQualityDIR;
			bool ResampleStrokes;
			long NumResampledPoints;
			bool ResampleStrokesGEF;
			long NumResampledPointsGEF;
			bool DisplayColours;

			class CStrokeMetrics{
				public:
					double BoxLength, Sagitta;
					bool ShortStroke, SmallAndFlat;
					CStrokeMetrics::CStrokeMetrics()
					{
						BoxLength= 0; Sagitta= 0;
						ShortStroke= false; SmallAndFlat= false;
					}
			};

			class CConicParams{
				public:
					double a, b, c, d, e, f;
					CConicParams::CConicParams()
					{
						a= 0; b= 0;c= 0; d= 0; e= 0; f=0;
					}
					CConicParams::CConicParams(double aa, double bb, double cc, double dd, double ee, double ff)
						:a (aa), b (bb), c (cc), d (dd), e (ee), f(ff)
					{
					}
					void CConicParams::PutParams(double aa, double bb, double cc, double dd, double ee, double ff)
					{
						a= aa; b= bb; c= cc; d= dd; e= ee; f= ff;
					}
					void CConicParams::GetParams(double &aa, double &bb, double &cc, double &dd, double &ee, double &ff)
					{
						aa= a; bb= b; cc= c; dd= d; ee= e; ff= f;
					}
			};

			bool IsEllipse(CConicParams phi);
			bool FivePointsConic(double p0[3], double p1[3], double p2[3],
								 double p3[3], double p4[3], 
								 CConicParams &phi);
			bool FitEllipse5P(CStroke InputStroke, CConicParams &phi);
			bool FitEllipse5PDQ(CStroke InputStroke, CConicParams &phi);
			bool FitEllipseDIR(CStroke InputStroke, CConicParams &phi);

			class SzpakData{
				public:
				bool use_pseudoinverse;
				bool theta_updated;
				double lambda;
				long k;
				double damping_multiplier;
				double gamma;
				long numberOfPoints;
				std::vector<std::vector<double>> F;
				std::vector<std::vector<double>> I;
				double alpha;
				std::vector<std::vector<double>> data_points;

				double tolDelta;
				double tolCost;
				double tolTheta;
				std::vector<std::vector<double>> cost;
				std::vector<std::vector<double>> t;
				std::vector<std::vector<double>> delta;

				std::vector<std::vector<double>> r;
				std::vector<std::vector<double>> jacobian_matrix;
				std::vector<std::vector<double>> jacobian_matrix_barrier;
				std::vector<std::vector<double>> jacobian_matrix_full;
				std::vector<std::vector<double>> H;
			};

			void normalise2dpts(std::vector<std::vector<double>> &pts,
								std::vector<std::vector<double>> &T);
			void LevenbergMarquardtStep(SzpakData &S);
			void LineSearchStep(SzpakData &S);
			void guaranteedEllipseFit(long numStroke, long numSubStroke,
									  std::vector<double> &theta,
									  std::vector<std::vector<double>> pts);
			bool FitEllipseGEF(CStroke InputStroke, long numStroke, long numSubStroke, CConicParams &phi);

			double DistancePointEllipse(const double e[2], const double y[2], double x[2]);
			POINT2D ClosestPointInEllipse(double Ra, double Rb, POINT2D P);
			void GetEndPoints(CStroke Stroke, CArc &Arc, bool EndPointsFitted);
			bool GetArc(CConicParams phi, CStroke Stroke, CArc &Arc, bool EndPointsFitted);

			void GetStrokeMetrics(CStroke Stroke, CStrokeMetrics &SM);
			bool GetConvexHull(CStroke &Stroke);

			bool IsSmallArc(CArc Arc);
			bool IsArcOverflow(CStroke Stroke, CArc Arc,
							   double BoxLength, double Sagitta);
			bool IsGapEnds(CStroke Stroke, CArc Arc);
			bool IsArcUnderflow(CStroke Stroke, CArc Arc,
							   double BoxLength, double Sagitta);
			bool IsBadFit5P(CStroke Stroke, CArc Arc, CStrokeMetrics SM);
			bool Fit5P(CStroke Stroke, CArc &Arc,
					   bool UseConvexHull5P, bool CheckQuality, CStrokeMetrics SM);
			bool IsBadFitDIR(CStroke Stroke, CArc Arc, CStrokeMetrics SM);
			bool FitDIR(CStroke Stroke, CArc &Arc,
						bool CheckQuality, CStrokeMetrics SM);
			bool FastFit(CStroke Stroke, CArc &Arc, CStrokeMetrics SM);
			bool FitGEF(CStroke Stroke, CArc &Arc, long numStroke, long numSubStroke);

			bool FitEllipseArc(CStroke Stroke, CArc &Arc, long numStroke, long numSubStroke);

			double GetStrokeArcWidth(CStroke &Stroke, CArc Arc);
			long SmoothStroke(CStroke &Stroke, CArc Arc,
							  double ToleranceWidth,
							  double widthInitial);
		public:
			CFitEllipse::CFitEllipse();
			CFitEllipse::CFitEllipse(double ATMin, double ATMax, double SPenalty,
									 long SCApproach,
									 long FEApproach,
									 long FESpeed,
									 bool UseCH5P,
									 bool Quality5P, bool QualityDIR,
									 bool RStrokes, long NumP,
									 bool RStrokesGEF, long NumPGEF,
									 bool DColours);
			CFitEllipse::~CFitEllipse();

			double GetMeritEllipse(CStroke &Stroke, long numStroke, long numSubStroke, CArc &Arc);
	};


#endif  // _FITELLIPSE_
