//+-----------------------------------------------------------------------------+
//| Copyright (C) 2013 REGEO (http://www.regeo.uji.es/)
//|
//| This program is free software
//| You can redistribute it and/or modify it under the terms of the GNU General Public License,
//| as published by the Free Software Foundation.
//| 
//| This program is distributed in the hope that it will be useful,
//| but WITHOUT ANY WARRANTY; without even the implied warranty of
//| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//+----------------------------------------------------------------------------*/

#ifndef _STROKE
	#define _STROKE

	class CStrokePoint
	// Used to store coordinates of points sampled when moving the cursor in sketch mode
	// The current time is also stored, to later evaluate sequences
	{
		private:
			double x, y, time;
		public:
			CStrokePoint();
			CStrokePoint(double cx, double cy, double ctime);
			CStrokePoint(const CStrokePoint &IP);
			~CStrokePoint();

			void PutPosition(double cx, double cy);
			POINT2D GetPosition();
			void PutTime(double ctime);
			double GetTime();
	};


	class CStroke
	// Stores information of a single stroke:
	//    StrokePoints  is a vector that contains pointers to the sequence of points
	//                  that belong to the same sketched stroke
	{
		private:
			std::vector<CStrokePoint> StrokePoints;
		public:
			CStroke();
			CStroke(const CStroke &IS);
			~CStroke();

			CStroke operator+(const CStroke& right) const;

			//Points edits
			//------------
			void Clear();
			void AddPoint(CStrokePoint Point);
			CStrokePoint GetPoint(long iPoint);
			void PutPoint(long Position, CStrokePoint Point);
			void RemovePoint(long iPoint);
			CStroke GetSubStroke(long PositionI, long PositionF);

			//Stroke metrics
			//--------------
			long GetNumPoints();
			RECTANGLE2D GetBounds();
			double GetLength();
			double GetLength(long numIntervals);
			double GetSlope180();
			double GetMaxSegment();
			POINT2D GetCentroid();
			std::vector<std::vector<double>> GetNormaliseMatrix();
			bool IsClockwise();
			void GetConvexHull();

			//Stroke edits
			//------------
			void Translate(POINT2D distance);
			void Rotate(POINT2D Centre, double angle);
			void Scale(POINT2D Centre, POINT2D Scale);
			void CleanEnds();
			void Filter(long FilterGap);
			bool Filter(double maxDistance);
			bool Resample(long NumResampledPoints);

			//Stroke cues
			//-----------
			long FindCorners(long StrokeCornersApproach);
			long NumCorners();

			//Corners
			std::vector <long> positions;

			bool grouped;
			bool segmented;

			//Merits
			double MeritLine;
			long NumLine;  //Drawing database line linked to this stroke

			double MeritArc;
			long NumArc;  //Drawing database arc linked to this stroke

			//--------------------------------------
			double worstOut_x;   //Used for display
			double worstOut_y;   //Used for display
			double t_Out;
			double worstIn_x;   //Used for display
			double worstIn_y;   //Used for display
			double t_In;
			//--------------------------------------

			void ClearMerits();
	};


#endif  // STROKE
