//+-----------------------------------------------------------------------------+
//| Copyright (C) 2013 REGEO (http://www.regeo.uji.es/)
//|
//| This program is free software
//| You can redistribute it and/or modify it under the terms of the GNU General Public License,
//| as published by the Free Software Foundation.
//| 
//| This program is distributed in the hope that it will be useful,
//| but WITHOUT ANY WARRANTY; without even the implied warranty of
//| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//+----------------------------------------------------------------------------*/

#ifndef TOOLS_GEOMETRY_VISITED
	#define TOOLS_GEOMETRY_VISITED

	const double Accuracy = 0.0000001;

	const double NUMBER_PI = 3.14159265359;
	#define DEGREES_TO_RADIANS(degrees) degrees*NUMBER_PI/180
	#define RADIANS_TO_DEGREES(radians) radians*180/NUMBER_PI

	#define MODULE_2D(A,B) sqrt(pow(A.x-B.x,2)+pow(A.y-B.y,2))
	//#define MODULE_3D(A,B) sqrt(pow(A.x-B.x,2)+pow(A.y-B.y,2)+pow(A.z-B.z,2))
	#define MODULE2D(v1x, v1y, v2x, v2y) sqrt(pow(v2x-v1x,2)+pow(v2y-v1y,2))
	//#define MODULE3D(v1x, v1y, v1z, v2x, v2y, v2z) sqrt(pow(v2x-v1x,2)+pow(v2y-v1y,2)+pow(v2z-v1z,2))

	//#define DOT_PRODUCT_2D(A,B) ( A.x*B.x + A.y*B.y )
	//#define DOT_PRODUCT_3D(A,B)( A.x*B.x + A.y*B.y + A.z*B.z )

	#define cross(a, b, ab) ab[0] = a[1]*b[2] - a[2]*b[1]; \
							ab[1] = a[2]*b[0] - a[0]*b[2]; \
							ab[2] = a[0]*b[1] - a[1]*b[0];


	class POINT2D
	{
		public:
			double x, y;

			POINT2D();
			POINT2D(const POINT2D& DP);
			POINT2D(double cx, double cy);
			~POINT2D();
			bool operator==(const POINT2D& right) const;
			bool operator!=(const POINT2D& right) const;
			POINT2D& operator+=(const POINT2D& right);
			POINT2D operator+(const POINT2D& right) const;
			POINT2D& operator-=(const POINT2D& right);
			POINT2D operator-(const POINT2D& right) const;
			POINT2D& operator*=(const POINT2D& right);
			POINT2D operator*(const POINT2D& right) const;
			POINT2D& operator/=(const POINT2D& right);
			POINT2D operator/(const POINT2D& right) const;

			POINT2D Rotate(POINT2D Center, double Angle);
			POINT2D Rotate(double Angle);
	};


	class SEGMENT2D
	{
		public:
			POINT2D p0, p1;

			SEGMENT2D();
			SEGMENT2D(const SEGMENT2D& DS);
			SEGMENT2D(POINT2D cp0, POINT2D cp1);
			~SEGMENT2D();
			bool operator==(const SEGMENT2D& right) const;
			bool operator!=(const SEGMENT2D& right) const;

			void Rotate(POINT2D Center, double Angle);
	};


	class RECTANGLE2D
	{
		public:
			double left, right, bottom, top;

			RECTANGLE2D();
			RECTANGLE2D(const RECTANGLE2D& DR);
			RECTANGLE2D(double cleft, double cright, double cbottom, double ctop);
			~RECTANGLE2D();
	};

		double GetAngle_2PI(POINT2D T, POINT2D H);
		double GetAngle_2PI(SEGMENT2D S);
		double GetAngle3Points(POINT2D P1, POINT2D P2, POINT2D P3);

		double GetDistancePointToSegment(POINT2D p0, POINT2D p1, POINT2D V,
										 POINT2D &Intersection);
		double GetDistancePointToSegment(POINT2D p0, POINT2D p1, POINT2D V);

		RECTANGLE2D GetBounds(std::vector <POINT2D> Points);


#endif  // TOOLS_GEOMETRY